@@ -10,13 +10,13 @@
 
 /*
 ** \file net.cpp
-** \version $Id: net.cpp 3735 2009-04-28 20:28:01Z edmanm $
 ** \brief Common network I/O and utility functions
 */
 
 #include "net.h"
 
 #include <QTcpSocket>
+#include <QLocalSocket>
 
 
 /** Attempts a connection to <b>host</b> on <b>port</b>. Returns true if the
@@ -33,3 +33,15 @@ net_test_connect(QHostAddress host, quint16 port, int timeout)
   return true;
 }
 
+bool
+socket_test_connect(QString server, int timeout)
+{
+  QLocalSocket sock;
+  sock.connectToServer(server);
+  if (!sock.waitForConnected(timeout)) {
+    return false;
+  }
+  sock.disconnectFromServer();
+  return true;
+}
+
