@@ -10,7 +10,6 @@
 
 /*
 ** \file MainWindow.cpp
-** \version $Id: MainWindow.cpp 4407 2010-08-26 16:55:53Z edmanm $
 ** \brief Main (hidden) window. Creates tray menu and child windows
 **
 ** Implements the main window. The main window is a hidden window that serves
@@ -118,6 +117,8 @@ MainWindow::MainWindow()
           this, SLOT(showHelpDialog(QString)));
   connect(_configDialog, SIGNAL(helpRequested(QString)),
           this, SLOT(showHelpDialog(QString)));
+  connect(_configDialog, SIGNAL(restartTor()),
+          this, SLOT(restart()));
 
   /* Create the actions that will go in the tray menu */
   createActions();
@@ -1000,10 +1001,17 @@ MainWindow::start()
   updateTorStatus(Starting);
 
   /* Check if Tor is already running separately */
-  if (net_test_connect(settings.getControlAddress(),
-                       settings.getControlPort())) {
-    started();
-    return;
+  if(settings.getControlMethod() == ControlMethod::Port) {
+    if (net_test_connect(settings.getControlAddress(),
+                        settings.getControlPort())) {
+      started();
+      return;
+    }
+  } else {
+    if (socket_test_connect(settings.getSocketPath())) {
+      started();
+      return;
+    }
   }
 
   /* Make sure the torrc we want to use really exists. */
@@ -1019,10 +1027,15 @@ MainWindow::start()
   if (!dataDirectory.isEmpty())
     args << "DataDirectory" << expand_filename(dataDirectory);
   
-  /* Add the intended control port value */
-  quint16 controlPort = settings.getControlPort();
-  if (controlPort)
-    args << "ControlPort" << QString::number(controlPort);
+  if(settings.getControlMethod() == ControlMethod::Port) {
+    /* Add the intended control port value */
+    quint16 controlPort = settings.getControlPort();
+    if (controlPort)
+      args << "ControlPort" << QString::number(controlPort);
+  } else {
+    QString path = settings.getSocketPath();
+    args << "ControlSocket" << path;
+  }
   
   /* Add the control port authentication arguments */
   switch (settings.getAuthenticationMethod()) {
@@ -1055,6 +1068,15 @@ MainWindow::start()
   _torControl->start(settings.getExecutable(), args);
 }
 
+/** Called when the user changes a setting that needs Tor restarting */
+void
+MainWindow::restart()
+{
+  if(_torControl->stop()) {
+    start();
+  }
+}
+
 /** Called when the Tor process fails to start, for example, because the path
  * specified to the Tor executable didn't lead to an executable. */
 void
@@ -1103,8 +1125,11 @@ MainWindow::started()
   /* Remember whether we started Tor or not */
   _isVidaliaRunningTor = _torControl->isVidaliaRunningTor();
   /* Try to connect to Tor's control port */
-  _torControl->connect(settings.getControlAddress(),
-                       settings.getControlPort());
+  if(settings.getControlMethod() == ControlMethod::Port)
+    _torControl->connect(settings.getControlAddress(),
+                        settings.getControlPort());
+  else
+    _torControl->connect(settings.getSocketPath());
   setStartupProgress(STARTUP_PROGRESS_CONNECTING, tr("Connecting to Tor"));
 }
 
