@@ -10,7 +10,6 @@
 
 /*
 ** \file Vidalia.cpp
-** \version $Id: Vidalia.cpp 4409 2010-08-26 17:30:56Z edmanm $
 ** \brief Main Vidalia QApplication object
 */
 
@@ -126,7 +125,7 @@ Vidalia::Vidalia(QStringList args, int &argc, char **argv)
   setStyle(_args.value(ARG_GUISTYLE));
 
   /* Creates a TorControl object, used to talk to Tor. */
-  _torControl = new TorControl();
+  _torControl = new TorControl(TorSettings().getControlMethod());
 
   /* If we were built with QSslSocket support, then populate the default
    * CA certificate store. */
