@@ -10,11 +10,11 @@
 
 /*
 ** \file AdvancedPage.cpp
-** \version $Id: AdvancedPage.cpp 4353 2010-07-14 15:50:50Z edmanm $
 ** \brief Advanced Tor and Vidalia configuration options
 */
 
 #include "AdvancedPage.h"
+#include "TorrcDialog.h"
 #include "Vidalia.h"
 #include "VMessageBox.h"
 #include "IpValidator.h"
@@ -60,12 +60,18 @@ AdvancedPage::AdvancedPage(QWidget *parent)
           this, SLOT(authMethodChanged(int)));
   connect(ui.chkRandomPassword, SIGNAL(toggled(bool)),
           ui.linePassword, SLOT(setDisabled(bool)));
+  connect(ui.btnEditTorConfig, SIGNAL(clicked()),
+          this, SLOT(displayTorrcDialog()));
+  connect(ui.rdoControlPort, SIGNAL(toggled(bool)), this, SLOT(toggleControl(bool)));
+  connect(ui.btnBrowseSocketPath, SIGNAL(clicked()), this, SLOT(browseSocketPath()));
 
   /* Hide platform specific features */
 #if defined(Q_WS_WIN)
 #if 0
   ui.grpService->setVisible(TorService::isSupported());
 #endif
+  /* Disable ControlSocket */
+  ui.rdoControlSocket->setEnabled(false);
 #endif
 }
 
@@ -111,12 +117,24 @@ AdvancedPage::revert()
 bool
 AdvancedPage::save(QString &errmsg)
 {
-  /* Validate the control listener address */
   QHostAddress controlAddress(ui.lineControlAddress->text());
-  if (controlAddress.isNull()) {
-    errmsg = tr("'%1' is not a valid IP address.")
-               .arg(ui.lineControlAddress->text());
-    return false; 
+  QString path(ui.lineSocketPath->text());
+
+  /* Validate the control settings */
+  if(ui.rdoControlPort->isChecked()) {
+    if (controlAddress.isNull()) {
+      errmsg = tr("'%1' is not a valid IP address.")
+                .arg(ui.lineControlAddress->text());
+      return false; 
+    }
+    _settings->setControlMethod(ControlMethod::Port);
+  } else {
+    QFileInfo finfo(path);
+    if(!finfo.exists()) {
+      errmsg = tr("ControlSocket path doesn't exist.");
+      return false;
+    }
+    _settings->setControlMethod(ControlMethod::Socket);
   }
   
   /* Validate the selected authentication options */
@@ -147,21 +165,25 @@ AdvancedPage::save(QString &errmsg)
 
   /* Only remember the torrc and datadir values if Vidalia started Tor, or
    * if the user changed the displayed values. */
-  if (!Vidalia::torControl()->isVidaliaRunningTor()) {
+  if (Vidalia::torControl()->isVidaliaRunningTor()) {
     QString torrc = ui.lineTorConfig->text();
-    if (torrc != _settings->getTorrc())
+    if (torrc != _settings->getTorrc()) {
       _settings->setTorrc(torrc);
+      QMessageBox::StandardButtons res = QMessageBox::question(this, tr("Warning"), 
+          tr("You changed torrc path, would you like to restart Tor?"),
+          QMessageBox::Yes | QMessageBox::No);
+      if(res == QMessageBox::Yes)
+        emit restartTor();
+    }
 
     QString dataDir = ui.lineTorDataDirectory->text();
     if (dataDir != _settings->getDataDirectory())
       _settings->setDataDirectory(dataDir);
-  } else {
-    _settings->setTorrc(ui.lineTorConfig->text());
-    _settings->setDataDirectory(ui.lineTorDataDirectory->text());
   }
 
   _settings->setControlAddress(controlAddress);
   _settings->setControlPort(ui.lineControlPort->text().toUShort());
+  _settings->setSocketPath(ui.lineSocketPath->text());
 
   _settings->setAuthenticationMethod(authMethod);
   _settings->setUseRandomPassword(ui.chkRandomPassword->isChecked());
@@ -193,6 +215,9 @@ AdvancedPage::load()
   ui.chkRandomPassword->setChecked(_settings->useRandomPassword());
   if (!ui.chkRandomPassword->isChecked())
     ui.linePassword->setText(_settings->getControlPassword());
+  ui.rdoControlPort->setChecked(_settings->getControlMethod() == ControlMethod::Port);
+  ui.rdoControlSocket->setChecked(_settings->getControlMethod() == ControlMethod::Socket);
+  ui.lineSocketPath->setText(_settings->getSocketPath());
 
 #if 0
 #if defined(Q_WS_WIN)
@@ -294,6 +319,22 @@ AdvancedPage::browseTorDataDirectory()
     ui.lineTorDataDirectory->setText(dataDir);
 }
 
+/** Opens a QFileDialog for the user to browse to or create a socket path to
+ * communicate to Tor */
+void
+AdvancedPage::browseSocketPath()
+{
+  QString start = QDir::currentPath();
+  if(!ui.lineSocketPath->text().isEmpty())
+    start = ui.lineSocketPath->text();
+  QString socketPath = QFileDialog::getOpenFileName(this,
+                      tr("Select a file to use for Tor socket path"),
+                      start);
+
+  if (!socketPath.isEmpty()) 
+    ui.lineSocketPath->setText(socketPath);
+}
+
 #if 0
 #if defined(Q_WS_WIN)
 /** Installs or removes the Tor service as necessary. */
@@ -329,3 +370,32 @@ AdvancedPage::setupService(bool useService)
 #endif
 #endif
 
+/** Called when the user presses the Edit current torrc button */
+void 
+AdvancedPage::displayTorrcDialog()
+{
+  TorrcDialog rcdialog(this);
+  rcdialog.exec();
+}
+
+void 
+AdvancedPage::toggleControl(bool)
+{
+  if(ui.rdoControlPort->isChecked()) {
+    ui.lblAddress->setEnabled(true);
+    ui.lineControlAddress->setEnabled(true);
+    ui.lineControlPort->setEnabled(true);
+    ui.lblPath->setEnabled(false);
+    ui.lineSocketPath->setEnabled(false);
+    ui.btnBrowseSocketPath->setEnabled(false);
+  } else {
+#if !defined(Q_OS_WIN32)
+    ui.lblAddress->setEnabled(false);
+    ui.lineControlAddress->setEnabled(false);
+    ui.lineControlPort->setEnabled(false);
+    ui.lblPath->setEnabled(true);
+    ui.lineSocketPath->setEnabled(true);
+    ui.btnBrowseSocketPath->setEnabled(true);
+#endif
+  }
+}
