@@ -10,7 +10,6 @@
 
 /*
 ** \file ServerPage.cpp
-** \version $Id: ServerPage.cpp 4143 2009-10-12 00:59:22Z edmanm $
 ** \brief Tor server configuration options
 */
 
@@ -102,6 +101,8 @@ ServerPage::ServerPage(QWidget *parent)
                               this, SLOT(copyBridgeIdentity()));
   connect(ui.lblBridgeUsage, SIGNAL(linkActivated(QString)),
                        this, SLOT(linkActivated(QString)));
+  connect(ui.lblWhatsThis, SIGNAL(linkActivated(QString)),
+                       this, SLOT(linkActivated(QString)));
 
   /* Set validators for address, mask and various port number fields */
   ui.lineServerNickname->setValidator(new NicknameValidator(this));
@@ -232,6 +233,9 @@ ServerPage::serverModeChanged(bool enabled)
   ui.chkPublishBridgeAddress->setVisible(bridgeEnabled);
   ui.lblBridgeUsage->setVisible(bridgeEnabled
                                   && Vidalia::torControl()->isConnected());
+
+  ui.lineDirPort->setEnabled(!bridgeEnabled);
+  ui.chkMirrorDirectory->setEnabled(!bridgeEnabled);
 }
 
 /** Returns true if the user has changed their server settings since the
@@ -291,10 +295,14 @@ ServerPage::save(QString &errmsg)
     _settings->setPublishServerDescriptor(ui.chkPublishBridgeAddress->isChecked());
 
   /* Save the rest of the server settings. */
-  _settings->setDirectoryMirror(ui.chkMirrorDirectory->isChecked());
   _settings->setNickname(ui.lineServerNickname->text());
   _settings->setORPort(ui.lineServerPort->text().toUInt());
-  _settings->setDirPort(ui.lineDirPort->text().toUInt());
+  if (!ui.rdoBridgeMode->isChecked()) {
+    _settings->setDirPort(ui.lineDirPort->text().toUInt());
+    _settings->setDirectoryMirror(ui.chkMirrorDirectory->isChecked());
+  } else {
+    _settings->setDirectoryMirror(false);
+  }
   _settings->setContactInfo(ui.lineServerContact->text());
   saveBandwidthLimits();
   saveExitPolicies();
@@ -550,6 +558,8 @@ ServerPage::linkActivated(const QString &url)
 {
   if (!url.compare("#bridgeUsage"))
     displayBridgeUsage();
+  else if(!url.compare("#bridgeHelp"))
+    emit helpRequested("bridges.about");
 }
 
 /** Retrieves bridge usage history from Tor, parses and validates it, and
@@ -558,15 +568,11 @@ void
 ServerPage::displayBridgeUsage()
 {
   QString info;
+  QMessageBox dlg(this);
 
   info = Vidalia::torControl()->getInfo("status/clients-seen").toString();
   if (info.isEmpty()) {
-    QMessageBox dlg(QMessageBox::Information, tr("No Recent Usage"),
-                    tr("No clients have used your relay recently."), 
-                    QMessageBox::Ok, this);
-    dlg.setInformativeText(tr("Leave your relay running so clients have "
-                              "a better chance of finding and using it."));
-    dlg.exec();
+    goto none;
   } else {
     QDateTime timeStarted;
     QHash<QString,int> countrySummary;
@@ -584,29 +590,47 @@ ServerPage::displayBridgeUsage()
     if (!timeStarted.isValid())
       goto err;
 
-    foreach (QString pair, keyvals.value("CountrySummary").split(",")) {
-      QStringList parts = pair.split("=");
-      if (parts.size() != 2)
-        goto err;
+    QStringList summary = keyvals.value("CountrySummary")
+                                 .split(",", QString::SkipEmptyParts);
+    if (summary.isEmpty()) {
+      goto none;
+    } else {
+      foreach (QString pair, summary) {
+        QStringList parts = pair.split("=");
+        if (parts.size() != 2)
+          goto err;
 
-      countrySummary.insert(parts.at(0).toUpper(), parts.at(1).toInt(&ok));
-      if (!ok)
-        goto err;
+        countrySummary.insert(parts.at(0).toUpper(), parts.at(1).toInt(&ok));
+        if (!ok)
+          goto err;
+      }
+
+      dlg.update(timeStarted, countrySummary);
+      dlg.exec();
     }
-
-    dlg.update(timeStarted, countrySummary);
-    dlg.exec();
   }
   return;
 
+none:
+  dlg.setIcon(QMessageBox::Information);
+  dlg.setWindowTitle(tr("No Recent Usage"));
+  dlg.setText(tr("No clients have used your relay recently."));
+  dlg.setInformativeText(tr("Leave your relay running so clients have "
+                            "a better chance of finding and using it."));
+  dlg.setStandardButtons(QMessageBox::Ok);
+  dlg.exec();
+  return;
+
 err:
-  QMessageBox dlg(QMessageBox::Warning, tr("Bridge History"),
-                  tr("Vidalia was unable to retrieve your bridge's usage "
-                     "history."), QMessageBox::Ok, this);
+  dlg.setIcon(QMessageBox::Warning);
+  dlg.setWindowTitle(tr("Bridge History"));
+  dlg.setText(tr("Vidalia was unable to retrieve your bridge's usage "
+                 "history."));
   dlg.setInformativeText(tr("Tor returned an improperly formatted "
                             "response when Vidalia requested your "
                             "bridge's usage history."));
   dlg.setDetailedText(tr("The returned response was: %1").arg(info));
+  dlg.setStandardButtons(QMessageBox::Ok);
   dlg.exec();
 }
 
